import yaml
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import GridSearchCV



def make_model(model_type, class_weights):
    model = None
    if model_type == 'RFClassifier':
        model = RandomForestClassifier(class_weight=class_weights)
    return model


def run_grid_search(X, y, model, param_grid, cv=5):
    grid_search = GridSearchCV(model,
                               param_grid=param_grid,
                               cv=cv,
                               verbose=2)
    grid_search.fit(X, y)
    return grid_search


def train_model(X_train, y_train, weights, model=None, config_path='./config/config.yaml'):
    config = yaml.safe_load(open(config_path, 'r'))
    model_type = config['Inputs']['model']
    if model is None:
        model = make_model(config['Inputs']['model'], class_weights=weights )
    print("SHAPE: ", X_train.shape, y_train.shape,)
    grid_search = run_grid_search(X_train, y_train, model, param_grid=config[model_type]['param_grid'])
    model.set_params(**grid_search.best_params_)
    model.fit(X_train, y_train)
    return model



import yaml

config_path = "../config/config.yaml"
with open(config_path, 'r') as f:
    config = yaml.safe_load(f)

model_dict = []
for model in config['models']:
    print(model)
    model_name = model
    model_class = model['class']
    print(model_class)
    # model_params = model['param_grid']
    # model_dict.append((model_name, model_class, model_params))


from sklearn.datasets import make_classification
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score
from sklearn.metrics import confusion_matrix
from sklearn.metrics import classification_report



from vtacML.pipeline import predict_from_best_pipeline
import pandas as pd
config_path = "../config/config.yaml"

data = pd.read_parquet('/vtacML/data/combined_qpo_vt_with_GRB.parquet')
data_faint = pd.read_parquet('/vtacML/data/combined_qpo_vt_faint_case_with_GRB_with_flags.parquet')
X_GRB = data[data['IS_GRB'] == 1]

y = predict_from_best_pipeline(X=X_GRB, config_file=config_path)

print(sum(y), len(y))





import pandas as pd
from imblearn.over_sampling import SMOTE

data = pd.read_parquet('/vtacML/data/combined_qpo_vt_with_GRB.parquet')
print(data['IS_GRB'].value_counts())
columns = ["MAGCAL_R0",
           "MAGCAL_B0",
           "MAGERR_R0",
           "MAGERR_B0",
           "MAGCAL_R1",
           "MAGCAL_B1",
           "MAGERR_R1",
           "MAGERR_B1",
           "MAGVAR_R1",
           "MAGVAR_B1",
           #    "CASE",
           #    "EFLAG_B1"
           "IS_GRB"
           ]

data = data[columns].dropna()


def resample(X):
    sm = SMOTE(sampling_strategy='minority', random_state=42)
    X_, y_ = sm.fit_resample(X.drop('IS_GRB', axis=1), X['IS_GRB'])
    X_resampled = pd.concat([pd.DataFrame(y_), pd.DataFrame(X_)], axis=1)
    return X_resampled


resampled_data = resample(data)
print(resampled_data['IS_GRB'].value_counts())



from setuptools import setup, find_packages

setup(
    name='vtacML',
    version='0.1.1',
    packages=find_packages(include=['testing', 'testing.*']),
    install_requires=[
        'numpy>=1.26.3',
        'matplotlib>=3.8.0',
        'pandas>=2.1.4',
        'scikit-learn>=1.3.0',
        'seaborn>=0.12.2',
        'yellowbrick>=1.5'
        # List your dependencies here
        # Example: 'numpy', 'pandas', 'scikit-learn',
    ],
    entry_points={
        'console_scripts': [
            # Define any command-line scripts here
            # Example: 'vtac_classifier=pipeline:main',
        ],
    },
    url='https://github.com/jerbeario/VTAC_ML',  # Replace with your project's URL
    license='MIT',
    author='Jeremy Palmerio',
    author_email='jeremypalmerio05@gmail.com',
    description='A machine learning pipeline to classify objects in VTAC dataset as GRB or not.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
